@@ -1,11 +1,11 @@
 use strict;
 use warnings;
-package JSON::Schema::Modern; # git description: v0.577-6-g4ee675c2
+package JSON::Schema::Modern; # git description: v0.578-18-ge0cf7239
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Validate data against a schema
 # KEYWORDS: JSON Schema validator data validation structure specification
 
-our $VERSION = '0.578';
+our $VERSION = '0.579';
 
 use 5.020;  # for fc, unicode_strings features
 use Moo;
@@ -29,6 +29,7 @@ use File::ShareDir 'dist_dir';
 use Module::Runtime qw(use_module require_module);
 use MooX::TypeTiny 0.002002;
 use Types::Standard 1.016003 qw(Bool Int Str HasMethods Enum InstanceOf HashRef Dict CodeRef Optional Slurpy ArrayRef Undef ClassName Tuple Map);
+use Digest::MD5 'md5';
 use Feature::Compat::Try;
 use JSON::Schema::Modern::Error;
 use JSON::Schema::Modern::Result;
@@ -97,22 +98,32 @@ has [qw(collect_annotations scalarref_booleans stringy_numbers strict)] => (
   isa => Bool,
 );
 
+my $core_types = Enum[qw(null object array boolean string number)];
+my @core_formats = qw(date-time date time duration email idn-email hostname idn-hostname ipv4 ipv6 uri uri-reference iri iri-reference uuid uri-template json-pointer relative-json-pointer regex);
+
+# { $format_name => { type => ..., sub => ... }, ... }
 has _format_validations => (
   is => 'bare',
-  isa => my $format_type = Dict[
-    (map +($_ => Optional[CodeRef]), qw(date-time date time duration email idn-email hostname idn-hostname ipv4 ipv6 uri uri-reference iri iri-reference uuid uri-template json-pointer relative-json-pointer regex)),
-    Slurpy[HashRef[Dict[type => Enum[qw(null object array boolean string number integer)], sub => CodeRef]]],
-  ],
+  isa => my $format_type = HashRef[Dict[
+      type => $core_types|ArrayRef[$core_types],
+      sub => CodeRef,
+    ]],
   init_arg => 'format_validations',
   lazy => 1,
   default => sub { {} },
 );
 
-sub _get_format_validation { $_[0]->{_format_validations}{$_[1]} }
+sub _get_format_validation ($self, $format) { $self->{_format_validations}{$format} }
 
-sub add_format_validation {
-  $format_type->({ @_[1..$#_] });
-  $_[0]->{_format_validations}{$_->[0]} = $_->[1] foreach pairs @_[1..$#_];
+sub add_format_validation ($self, $format, $definition) {
+  $definition = { type => 'string', sub => $definition } if not is_plain_hashref($definition);
+  $format_type->({ $format => $definition });
+
+  # all core formats are of type string (so far); changing type of custom format is permitted
+  croak "Type for override of format $format does not match original type"
+    if any { $format eq $_ } @core_formats and $definition->{type} ne 'string';
+
+  $self->{_format_validations}{$format} = $definition;
 }
 
 around BUILDARGS => sub ($orig, $class, @args) {
@@ -121,6 +132,14 @@ around BUILDARGS => sub ($orig, $class, @args) {
     if ($args->{output_format}//'') eq 'strict_basic'
       and ($args->{specification_version}//'') ne 'draft2019-09';
 
+  croak 'collect_annotations cannot be used with specification_version draft7'
+    if $args->{collect_annotations} and ($args->{specification_version}//'') eq 'draft7';
+
+  $args->{format_validations} = +{
+    map +($_->[0] => is_plain_hashref($_->[1]) ? $_->[1] : +{ type => 'string', sub => $_->[1] }),
+      pairs $args->{format_validations}->%*
+  } if $args->{format_validations};
+
   return $args;
 };
 
@@ -159,13 +178,13 @@ sub add_schema {
   }
 
   if (not grep refaddr($_->{document}) == refaddr($document), $self->_canonical_resources) {
-    my $schema_content = $document->_serialized_schema
-      // $document->_serialized_schema($self->_json_decoder->encode($document->schema));
+    my $schema_checksum = $document->_checksum
+      // $document->_checksum(md5($self->_json_decoder->encode($document->schema)));
 
     if (my $existing_doc = first {
-          my $existing_content = $_->_serialized_schema
-            // $_->_serialized_schema($self->_json_decoder->encode($_->schema));
-          $existing_content eq $schema_content
+          my $existing_checksum = $_->_checksum
+            // $_->_checksum(md5($self->_json_decoder->encode($_->schema)));
+          $existing_checksum eq $schema_checksum
         } uniqint map $_->{document}, $self->_canonical_resources) {
       # we already have this schema content in another document object.
       $document = $existing_doc;
@@ -355,13 +374,6 @@ sub evaluate ($self, $data, $schema_reference, $config_override = {}) {
       } qw(validate_formats validate_content_schemas short_circuit collect_annotations scalarref_booleans stringy_numbers strict)),
     };
 
-    if ($state->{validate_formats}) {
-      $state->{vocabularies} = [
-        map s/^JSON::Schema::Modern::Vocabulary::Format\KAnnotation$/Assertion/r, $state->{vocabularies}->@*
-      ];
-      require JSON::Schema::Modern::Vocabulary::FormatAssertion;
-    }
-
     # we're going to set collect_annotations during evaluation when we see an unevaluated* keyword,
     # but after we pass to a new data scope we'll clear it again.. unless we've got the config set
     # globally for the entire evaluation, so we store that value in a high bit.
@@ -1110,8 +1122,11 @@ sub FREEZE ($self, $serializer) {
 sub THAW ($class, $serializer, $data) {
   my $self = bless($data, $class);
 
-  # load all vocabulary classes
-  require_module($_) foreach uniq map $_->{vocabularies}->@*, $self->_canonical_resources;
+  # load all vocabulary classes, both those used by loaded schemas, as well as all the core modules
+  require_module($_)
+    foreach uniq(
+      (map $_->{vocabularies}->@*, $self->_canonical_resources),
+      (map $_->[1], values $self->__vocabulary_classes->%*));
 
   return $self;
 }
@@ -1132,7 +1147,7 @@ JSON::Schema::Modern - Validate data against a schema
 
 =head1 VERSION
 
-version 0.578
+version 0.579
 
 =head1 SYNOPSIS
 
@@ -1213,6 +1228,12 @@ other, or badly-written schemas that could be optimized. Defaults to 50.
 When true, the C<format> keyword will be treated as an assertion, not merely an annotation. Defaults
 to true when specification_version is draft7, and false for all other versions, but this may change in the future.
 
+Note that the use of a format that does not have a defined handler will B<not> be interpreted as an
+error in this mode; instead, the undefined format will simply be ignored. If you instead want this
+to be treated as an evaluation error, you must define a custom schema dialect that uses the
+format-assertion vocabulary (available in specification version C<draft2020-12>) and reference it in
+your schema with the C<$schema> keyword.
+
 =head2 format_validations
 
 =for stopwords subref
@@ -1246,6 +1267,7 @@ fixed for the next draft.
 
 When true, annotations are collected from keywords that produce them, when validation succeeds.
 These annotations are available in the returned result (see L<JSON::Schema::Modern::Result>).
+Not operational when L</specification_version> is C<draft7>.
 Defaults to false.
 
 =head2 scalarref_booleans
@@ -1294,6 +1316,10 @@ C<minimum>
 
 C<exclusiveMinimum>
 
+=item *
+
+C<format> (for formats defined to validate numbers)
+
 =back
 
 This allows you to write a schema like this (which validates a string representing an integer):
@@ -1526,13 +1552,20 @@ otherwise returns the L<JSON::Schema::Modern::Document> that contains the added
 
 =head2 add_format_validation
 
+  $js->add_format_validation(all_lc => sub ($value) { lc($value) eq $value });
+
 =for comment we are the nine Eleven Deniers
 
+or
+
   $js->add_format_validation(no_nines => { type => 'number', sub => sub ($value) { $value =~ m/^[0-8]$$/ });
 
-Adds support for a custom format. The data type that this format applies to must be supplied; all
-values of any other type will automatically be deemed to be valid, and will not be passed to the
-subref.
+Adds support for a custom format. If not supplied, the data type(s) that this format applies to
+defaults to string; all values of any other type will automatically be deemed to be valid, and will
+not be passed to the subref.
+
+Additionally, you can redefine the definition for any core format (see L</Format Validation>), but
+the data type(s) supported by that format may not be changed.
 
 Be careful to not mutate the type of the value while checking it -- for example, if it is a string,
 do not apply arithmetic operators to it -- or subsequent type checks on this value may fail.
@@ -1825,6 +1858,10 @@ detect potentially pathological constructs that may pose a security risk, either
 service or by allowing exposure to the internals of your application. B<DO NOT USE SCHEMAS FROM
 UNTRUSTED SOURCES.>
 
+(In particular, see vulnerability
+L<perl5363delta/CVE-2023-47038-Write-past-buffer-end-via-illegal-user-defined-Unicode-property>,
+which is closed in Perl releases 5.34.3, 5.36.3 and 5.38.1.)
+
 =head1 SEE ALSO
 
 =for stopwords OpenAPI
