@@ -4,7 +4,7 @@ package JSON::Schema::Modern::Result;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Contains the result of a JSON Schema evaluation
 
-our $VERSION = '0.578';
+our $VERSION = '0.579';
 
 use 5.020;
 use Moo;
@@ -38,7 +38,7 @@ use constant { true => JSON::PP::true, false => JSON::PP::false };
 has valid => (
   is => 'ro',
   isa => Bool,
-  coerce => sub { !!$_[0] },
+  coerce => 1,
   required => 1,
 );
 sub result { goto \&valid } # backcompat only
@@ -242,7 +242,7 @@ JSON::Schema::Modern::Result - Contains the result of a JSON Schema evaluation
 
 =head1 VERSION
 
-version 0.578
+version 0.579
 
 =head1 SYNOPSIS
 
