@@ -4,7 +4,7 @@ package JSON::Schema::Modern::Utilities;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Internal utilities for JSON::Schema::Modern
 
-our $VERSION = '0.578';
+our $VERSION = '0.579';
 
 use 5.020;
 use strictures 2;
@@ -52,6 +52,7 @@ our @EXPORT_OK = qw(
 use JSON::PP ();
 use constant { true => JSON::PP::true, false => JSON::PP::false };
 
+# supports the six core types, plus integer (which is also a number)
 sub is_type ($type, $value) {
   if ($type eq 'null') {
     return !(defined $value);
@@ -92,6 +93,7 @@ sub is_type ($type, $value) {
   return ref($value) eq $type;
 }
 
+# returns one of the six core types, plus integer
 sub get_type ($value) {
   return 'object' if is_plain_hashref($value);
   return 'boolean' if is_bool($value);
@@ -237,7 +239,7 @@ sub E ($state, $error_string, @args) {
     or ($uri->fragment // '') eq $keyword_location and $uri->clone->fragment(undef) eq '';
 
   push $state->{errors}->@*, JSON::Schema::Modern::Error->new(
-    depth => $state->{depth},
+    depth => $state->{depth} // 0,
     keyword => $state->{keyword},
     instance_location => $state->{data_path},
     keyword_location => $keyword_location,
@@ -276,7 +278,7 @@ sub A ($state, $annotation) {
     .jsonp($state->{schema_path}, $state->{keyword}, delete $state->{_schema_path_suffix});
 
   push $state->{annotations}->@*, {
-    depth => $state->{depth},
+    depth => $state->{depth} // 0,
     keyword => $state->{keyword},
     instance_location => $state->{data_path},
     keyword_location => $keyword_location,
@@ -383,7 +385,7 @@ JSON::Schema::Modern::Utilities - Internal utilities for JSON::Schema::Modern
 
 =head1 VERSION
 
-version 0.578
+version 0.579
 
 =head1 SYNOPSIS
 
