@@ -4,7 +4,7 @@ package JSON::Schema::Modern::Vocabulary::FormatAssertion;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Implementation of the JSON Schema Format-Assertion vocabulary
 
-our $VERSION = '0.578';
+our $VERSION = '0.579';
 
 use 5.020;
 use Moo;
@@ -15,8 +15,11 @@ use if "$]" >= 5.022, experimental => 're_strict';
 no if "$]" >= 5.031009, feature => 'indirect';
 no if "$]" >= 5.033001, feature => 'multidimensional';
 no if "$]" >= 5.033006, feature => 'bareword_filehandles';
-use JSON::Schema::Modern::Utilities qw(is_type E A assert_keyword_type abort);
+use JSON::Schema::Modern::Utilities qw(get_type E A assert_keyword_type abort);
 use Feature::Compat::Try;
+use List::Util 'any';
+use Ref::Util 0.100 'is_plain_arrayref';
+use Scalar::Util 'looks_like_number';
 use namespace::clean;
 
 with 'JSON::Schema::Modern::Vocabulary';
@@ -175,20 +178,14 @@ sub keywords {
   $formats_by_spec_version{'draft2020-12'} = [$formats_by_spec_version{draft7}->@*, qw(duration uuid)];
 
   sub _get_default_format_validation ($class, $state, $format) {
-    return $formats->{$format}
+    # all core formats are of type string (so far)
+    return { type => 'string', sub => $formats->{$format} }
       if grep $format eq $_, $formats_by_spec_version{$state->{spec_version}}->@*;
   }
 }
 
-sub _traverse_keyword_format ($class, $schema, $state) {
-  return if not assert_keyword_type($state, $schema, 'string');
-  return 1;
-}
-
-sub _eval_keyword_format ($class, $data, $schema, $state) {
-  abort($state, 'unimplemented format "%s"', $schema->{format})
-    if $schema->{format} eq 'uri-template';
-
+# common code between FormatAnnotation+validate_formats and FormatAssertion paths
+sub _get_format_definition ($class, $schema, $state) {
   try {
     if ($schema->{format} eq 'date-time' or $schema->{format} eq 'date') {
       require Time::Moment;
@@ -211,20 +208,51 @@ sub _eval_keyword_format ($class, $data, $schema, $state) {
     abort($state, 'EXCEPTION: cannot validate with format "%s": %s', $schema->{format}, $e);
   }
 
-  # first check the subrefs from JSON::Schema::Modern->new(format_validations => { ... })
-  # and fall back to the default formats, which are all defined only for strings
-  my $evaluator_spec = $state->{evaluator}->_get_format_validation($schema->{format});
-  my $default_spec = $class->_get_default_format_validation($state, $schema->{format});
+  return $state->{evaluator}->_get_format_validation($schema->{format})
+    // $class->_get_default_format_validation($state, $schema->{format});
+}
 
-  my $spec =
-    $evaluator_spec ? ($default_spec ? +{ type => 'string', sub => $evaluator_spec } : $evaluator_spec)
-      : $default_spec ? +{ type => 'string', sub => $default_spec }
-      : undef;
+sub _traverse_keyword_format ($class, $schema, $state) {
+  return if not assert_keyword_type($state, $schema, 'string');
+  return E($state, 'unimplemented format "%s"', $schema->{format})
+    if $schema->{format} eq 'uri-template';
 
+  return E($state, 'unimplemented custom format "%s"', $schema->{format})
+    if not grep $state->{spec_version} eq $_, qw(draft7 draft2019-09)
+      and not $class->_get_default_format_validation($state, $schema->{format})
+      and not $state->{evaluator}->_get_format_validation($schema->{format});
+
+  return 1;
+}
+
+# Note that this method is only callable in draft2020-12 and later, because this vocabulary does not
+# exist in previous versions
+sub _eval_keyword_format ($class, $data, $schema, $state) {
   A($state, $schema->{format});
-  return E($state, 'not a valid %s', $schema->{format}) if $spec and is_type($spec->{type}, $data)
-    and not $spec->{sub}->($data);
 
+  # §7.2.2 (draft2020-12) "When the Format-Assertion vocabulary is declared with a value of true,
+  # implementations MUST provide full validation support for all of the formats defined by this
+  # specification. Implementations that cannot provide full validation support MUST refuse to
+  # process the schema."
+  abort($state, 'unimplemented format "%s"', $schema->{format})
+    if $schema->{format} eq 'uri-template';
+
+  my $spec = $class->_get_format_definition($schema, $state);
+
+  # §7.2.3 (draft2020-12) "When the Format-Assertion vocabulary is specified, implementations MUST
+  # fail upon encountering unknown formats."
+  abort($state, 'unimplemented custom format "%s"', $schema->{format}) if not $spec;
+
+  my $type = get_type($data);
+  $type = 'number' if $type eq 'integer';
+
+  return 1 if
+    not is_plain_arrayref($spec->{type}) ? any { $type eq $_ } $spec->{type}->@* : $type eq $spec->{type}
+    and not ($state->{stringy_numbers} and $type eq 'string'
+      and is_plain_arrayref($spec->{type}) ? any { $_ eq 'number' } $spec->{type}->@* : $spec->{type} eq 'number'
+      and looks_like_number($data));
+
+  return E($state, 'not a valid %s', $schema->{format}) if not $spec->{sub}->($data);
   return 1;
 }
 
@@ -242,7 +270,7 @@ JSON::Schema::Modern::Vocabulary::FormatAssertion - Implementation of the JSON S
 
 =head1 VERSION
 
-version 0.578
+version 0.579
 
 =head1 DESCRIPTION
 
